/* JOHO KISO A                     */
/* Yoshinori Hayakawa  16-JUL-2012 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define BUFSIZE 256

#define POW2(x) ((x)*(x))

#define YES 1
#define NO 0

int read_csv(char *filename, float *data)
{
  FILE *fp ;
  char line[BUFSIZE],buf[BUFSIZE] ;
  int n,started=NO ;
  int year,mon,day,hour,min,p ;
  fp = fopen(filename,"r") ;
  if (fp==NULL) {
    fprintf(stderr,"cannot open file: %s\n",filename) ;
    return 0 ;
  }
  fprintf(stderr,"reading: %s\n",filename) ;
  while (1) {
    if (fgets(line, BUFSIZE-1, fp)==NULL) break ;
    if ( !strncmp(line,"DATE",4) ) {
      started = YES ; n=-1 ;
    }
    if (started) {
      if (n>=0) {
	sscanf(line,"%d/%d/%d,%d:%d,%d%[^\n]",&year,&mon,&day,&hour,&min,&p,buf) ;
	data[n]=(float)p ;
      }
      n++ ;
    }
  }
  fclose(fp) ;
  fprintf(stderr,"%d points\n",n) ;
  return n ;
}

#include <time.h>

/* year1/mon1/day1 hour1:00 - year0/mon0/day0 hour0:00 */
int diff_in_hour(int year0, int mon0, int day0, int hour0,
	     int year1, int mon1, int day1, int hour1) 
{
  struct tm date0, date1 ;
  time_t time0,time1 ;
  date0.tm_sec=0 ;  date0.tm_min=0 ;  date0.tm_hour=hour0 ;  
  date0.tm_mday=day0 ; date0.tm_mon=mon0-1 ; date0.tm_year=year0-1900 ; date0.tm_isdst=-1 ;
  date1.tm_sec=0 ;  date1.tm_min=0 ;  date1.tm_hour=hour1 ;  
  date1.tm_mday=day1 ; date1.tm_mon=mon1-1 ; date1.tm_year=year1-1900 ; date1.tm_isdst=-1 ;
  time0 = mktime(&date0) ;  time1 = mktime(&date1) ;
  return (int)difftime(time1,time0)/3600 ;
}

/* year0/mon0/day0 0:00 + ofshours */
void print_date_plus_hour(int year0, int mon0, int day0, int ofshours)
{
  struct tm date0, *d1 ;
  time_t time0 ;
  date0.tm_sec=0 ;  date0.tm_min=0 ;  date0.tm_hour=0 ;
  date0.tm_mday=day0 ; date0.tm_mon=mon0-1 ; date0.tm_year=year0-1900 ; date0.tm_isdst=-1 ;
  time0 = mktime(&date0) + (time_t)(3600*ofshours) ;
  d1 = localtime(&time0) ;
  printf("%d/%02d/%02d %2d:00",d1->tm_year+1900, d1->tm_mon+1, d1->tm_mday, d1->tm_hour) ;
}

/* 0:Sun .... 6:Sat */
int day_of_the_week(int year, int month, int day)
{
  if (month == 1 || month == 2 ) {
    year--;
    month += 12;
  }
  return (year + year/4 - year/100 + year/400 + (13*month+8)/5 + day) % 7 ;
}

#define MMAX 1000

static double R_[MMAX+1],mean_=0 ;

double Rx(int n, float *data) {
  double s ;
  int i ;
  for (i=0,s=0 ; i<n ; i++) s += data[i] ;
  return s/n ;
}

double Rxx(int m, int n, float *data)
{
  int i ;
  double s2 ;
  if (m>=n) return 0.0 ;
  for (i=0,s2=0 ; i<n-m; i++) {
    s2 += (data[i] - mean_) * (data[i+m] - mean_) ;
  }
  return s2/(n-m) ;
}

/* Levinson-Durbin method */
static double coef2_[MMAX+1] ;
float arcoef(int p, int n, float *data, float *coef)
{
  double sm,sigma2 ;
  int i,j,m ;
  if (p>MMAX) { fprintf(stderr,"error: param(%d) exceeds %d.\n",p,MMAX) ; return 0 ; }
  srand48(0) ;
  mean_ = Rx(n,data) ;
  R_[0] = Rxx(0,n,data) ;
  R_[1] = Rxx(1,n,data) ;
  coef[0] = R_[1]/R_[0] ;
  sigma2 = R_[0] - POW2(R_[1])/R_[0] ;
  for (m=2 ; m<=p ; m++) {
    R_[m] = Rxx(m,n,data) ;
    for (j=0,sm=0 ; j<m-1 ; j++) sm += coef[j]*R_[m-j-1] ;
    coef[m-1] = (R_[m] - sm)/sigma2 ;
    for (j=0 ; j<m-1 ; j++) coef2_[j] = coef[j] - coef[m-1]*coef[m-j-2] ;
    sigma2 = sigma2*(1.0-POW2(coef[m-1])) ;
    for (j=0 ; j<m-1 ; j++) coef[j] = coef2_[j] ;
  }
  for (i=0,sm=0 ; i<p ; i++) sm += coef[i] ;
  coef[p] =  mean_*(1 - sm) ;
  return sqrt(sigma2) ;
}

#define DRAND drand48()
float dgauss()
{
  double v1,v2,r,fac ;
  do {
    v1 = 2.0*DRAND-1.0 ;
    v2 = 2.0*DRAND-1.0 ;
    r = v1*v1 + v2*v2 ;
  } while ( r >= 1.0 ) ;
  fac = sqrt(-2.0 * log(r)/r) ;
  return (float)(v2*fac) ;
} 
